@@ -8,8 +8,6 @@ import pathlib, sys,os
 sys.path.append(str(pathlib.Path(__file__).parent))
 
 import utils_ 
-utils_.load_model()
-
 
 
 import background_subtraction 
@@ -22,7 +20,7 @@ import objects_classes
 import post_process 
 
 import show_results
-import main 
+import core
 
 
 
